@@ -14,15 +14,16 @@ from django.conf import settings
 from django.db import models
 from django.db.models.query import QuerySet
 from django.utils.encoding import smart_str
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 from django.utils.text import get_text_list
 from django.contrib import admin
 
 from django_extensions.admin.widgets import ForeignKeySearchInput
 
 
-class ForeignKeyAutocompleteAdminMixin(object):
-    """Admin class for models using the autocomplete feature.
+class ForeignKeyAutocompleteAdminMixin:
+    """
+    Admin class for models using the autocomplete feature.
 
     There are two additional fields:
        - related_search_fields: defines fields of managed model that
@@ -51,7 +52,7 @@ class ForeignKeyAutocompleteAdminMixin(object):
     autocomplete_limit = getattr(settings, 'FOREIGNKEY_AUTOCOMPLETE_LIMIT', None)
 
     def get_urls(self):
-        from django.conf.urls import url
+        from django.urls import path
 
         def wrap(view):
             def wrapper(*args, **kwargs):
@@ -59,13 +60,13 @@ class ForeignKeyAutocompleteAdminMixin(object):
             return update_wrapper(wrapper, view)
 
         return [
-            url(r'foreignkey_autocomplete/$', wrap(self.foreignkey_autocomplete),
+            path('foreignkey_autocomplete/', wrap(self.foreignkey_autocomplete),
                 name='%s_%s_autocomplete' % (self.model._meta.app_label, self.model._meta.model_name))
-        ] + super(ForeignKeyAutocompleteAdminMixin, self).get_urls()
+        ] + super().get_urls()
 
     def foreignkey_autocomplete(self, request):
         """
-        Searches in the fields of the given related model and returns the
+        Search in the fields of the given related model and returns the
         result as a simple string to be used by the jQuery Autocomplete plugin
         """
         query = request.GET.get('q', None)
@@ -77,10 +78,7 @@ class ForeignKeyAutocompleteAdminMixin(object):
         try:
             to_string_function = self.related_string_functions[model_name]
         except KeyError:
-            if six.PY3:
-                to_string_function = lambda x: x.__str__()
-            else:
-                to_string_function = lambda x: x.__unicode__()
+            to_string_function = lambda x: x.__str__()
 
         if search_fields and app_label and model_name and (query or object_pk):
             def construct_search(field_name):
@@ -125,10 +123,13 @@ class ForeignKeyAutocompleteAdminMixin(object):
         return HttpResponseNotFound()
 
     def get_related_filter(self, model, request):
-        """Given a model class and current request return an optional Q object
+        """
+        Given a model class and current request return an optional Q object
         that should be applied as an additional filter for autocomplete query.
         If no additional filtering is needed, this method should return
-        None."""
+        None.
+        """
+        return None
 
     def get_help_text(self, field_name, model_name):
         searchable_fields = self.related_search_fields.get(field_name, None)
@@ -142,7 +143,7 @@ class ForeignKeyAutocompleteAdminMixin(object):
 
     def formfield_for_dbfield(self, db_field, **kwargs):
         """
-        Overrides the default widget for Foreignkey fields if they are
+        Override the default widget for Foreignkey fields if they are
         specified in the related_search_fields class attribute.
         """
         if isinstance(db_field, models.ForeignKey) and db_field.name in self.related_search_fields:
@@ -151,7 +152,7 @@ class ForeignKeyAutocompleteAdminMixin(object):
                 help_text = six.u('%s %s' % (kwargs['help_text'], help_text))
             kwargs['widget'] = ForeignKeySearchInput(db_field.remote_field, self.related_search_fields[db_field.name])
             kwargs['help_text'] = help_text
-        return super(ForeignKeyAutocompleteAdminMixin, self).formfield_for_dbfield(db_field, **kwargs)
+        return super().formfield_for_dbfield(db_field, **kwargs)
 
 
 class ForeignKeyAutocompleteAdmin(ForeignKeyAutocompleteAdminMixin, admin.ModelAdmin):
