@@ -4,12 +4,9 @@ from six.moves import urllib
 from django import forms
 from django.contrib.admin.sites import site
 from django.contrib.admin.widgets import ForeignKeyRawIdWidget
-from django.contrib.admin.templatetags.admin_static import static
-try:
-    from django.urls import reverse
-except ImportError:
-    from django.core.urlresolvers import reverse
 from django.template.loader import render_to_string
+from django.templatetags.static import static
+from django.urls import reverse
 from django.utils.safestring import mark_safe
 from django.utils.text import Truncator
 
@@ -46,7 +43,7 @@ class ForeignKeySearchInput(ForeignKeyRawIdWidget):
 
     def __init__(self, rel, search_fields, attrs=None):
         self.search_fields = search_fields
-        super(ForeignKeySearchInput, self).__init__(rel, site, attrs)
+        super().__init__(rel, site, attrs)
 
     def render(self, name, value, attrs=None, renderer=None):
         if attrs is None:
