@@ -23,17 +23,17 @@ except ImportError:
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
-from django.db.models import DateTimeField, CharField, SlugField
+from django.db.models import DateTimeField, CharField, SlugField, Q
 from django.db.models.constants import LOOKUP_SEP
 from django.template.defaultfilters import slugify
 from django.utils.crypto import get_random_string
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
 
 
 MAX_UNIQUE_QUERY_ATTEMPTS = getattr(settings, 'EXTENSIONS_MAX_UNIQUE_QUERY_ATTEMPTS', 100)
 
 
-class UniqueFieldMixin(object):
+class UniqueFieldMixin:
 
     def check_is_bool(self, attrname):
         if not isinstance(getattr(self, attrname), bool):
@@ -66,9 +66,22 @@ class UniqueFieldMixin(object):
                 for param in params:
                     kwargs[param] = getattr(model_instance, param, None)
 
+        # for support django 2.2+
+        query = Q()
+        constraints = getattr(model_instance._meta, 'constraints', None)
+        if constraints:
+            for constraint in constraints:
+                if self.attname in constraint.fields:
+                    condition = {
+                        field: getattr(model_instance, field, None)
+                        for field in constraint.fields
+                        if field != self.attname
+                    }
+                    query &= Q(**condition)
+
         new = six.next(iterator)
         kwargs[self.attname] = new
-        while not new or queryset.filter(**kwargs):
+        while not new or queryset.filter(query, **kwargs):
             new = six.next(iterator)
             kwargs[self.attname] = new
         setattr(model_instance, self.attname, new)
@@ -157,7 +170,7 @@ class AutoSlugField(UniqueFieldMixin, SlugField):
         self.allow_duplicates = kwargs.pop('allow_duplicates', False)
         self.check_is_bool('allow_duplicates')
         self.max_unique_query_attempts = kwargs.pop('max_unique_query_attempts', MAX_UNIQUE_QUERY_ATTEMPTS)
-        super(AutoSlugField, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def _slug_strip(self, value):
         """
@@ -232,7 +245,7 @@ class AutoSlugField(UniqueFieldMixin, SlugField):
             setattr(model_instance, self.attname, slug)
             return slug
 
-        return super(AutoSlugField, self).find_unique(
+        return self.find_unique(
             model_instance, slug_field, self.slug_generator(original_slug, start))
 
     def get_slug_fields(self, model_instance, lookup_value):
@@ -256,14 +269,14 @@ class AutoSlugField(UniqueFieldMixin, SlugField):
         return attr
 
     def pre_save(self, model_instance, add):
-        value = force_text(self.create_slug(model_instance, add))
+        value = force_str(self.create_slug(model_instance, add))
         return value
 
     def get_internal_type(self):
         return "SlugField"
 
     def deconstruct(self):
-        name, path, args, kwargs = super(AutoSlugField, self).deconstruct()
+        name, path, args, kwargs = super().deconstruct()
         kwargs['populate_from'] = self._populate_from
         if not self.separator == six.u('-'):
             kwargs['separator'] = self.separator
@@ -333,13 +346,19 @@ class RandomCharField(UniqueFieldMixin, CharField):
         if 'unique' not in kwargs:
             kwargs['unique'] = False
 
-        super(RandomCharField, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def random_char_generator(self, chars):
         for i in range(self.max_unique_query_attempts):
             yield ''.join(get_random_string(self.length, chars))
         raise RuntimeError('max random character attempts exceeded (%s)' % self.max_unique_query_attempts)
 
+    def in_unique_together(self, model_instance):
+        for params in model_instance._meta.unique_together:
+            if self.attname in params:
+                return True
+        return False
+
     def pre_save(self, model_instance, add):
         if not add and getattr(model_instance, self.attname) != '':
             return getattr(model_instance, self.attname)
@@ -360,12 +379,12 @@ class RandomCharField(UniqueFieldMixin, CharField):
             population += string.punctuation
 
         random_chars = self.random_char_generator(population)
-        if not self.unique:
+        if not self.unique and not self.in_unique_together(model_instance):
             new = six.next(random_chars)
             setattr(model_instance, self.attname, new)
             return new
 
-        return super(RandomCharField, self).find_unique(
+        return self.find_unique(
             model_instance,
             model_instance._meta.get_field(self.attname),
             random_chars,
@@ -375,7 +394,7 @@ class RandomCharField(UniqueFieldMixin, CharField):
         return "CharField"
 
     def deconstruct(self):
-        name, path, args, kwargs = super(RandomCharField, self).deconstruct()
+        name, path, args, kwargs = super().deconstruct()
         kwargs['length'] = self.length
         del kwargs['max_length']
         if self.lowercase is True:
@@ -410,7 +429,7 @@ class CreationDateTimeField(DateTimeField):
         return "DateTimeField"
 
     def deconstruct(self):
-        name, path, args, kwargs = super(CreationDateTimeField, self).deconstruct()
+        name, path, args, kwargs = super().deconstruct()
         if self.editable is not False:
             kwargs['editable'] = True
         if self.blank is not True:
@@ -437,7 +456,7 @@ class ModificationDateTimeField(CreationDateTimeField):
         return "DateTimeField"
 
     def deconstruct(self):
-        name, path, args, kwargs = super(ModificationDateTimeField, self).deconstruct()
+        name, path, args, kwargs = super().deconstruct()
         if self.auto_now is not False:
             kwargs['auto_now'] = True
         return name, path, args, kwargs
@@ -445,14 +464,14 @@ class ModificationDateTimeField(CreationDateTimeField):
     def pre_save(self, model_instance, add):
         if not getattr(model_instance, 'update_modified', True):
             return getattr(model_instance, self.attname)
-        return super(ModificationDateTimeField, self).pre_save(model_instance, add)
+        return super().pre_save(model_instance, add)
 
 
 class UUIDVersionError(Exception):
     pass
 
 
-class UUIDFieldMixin(object):
+class UUIDFieldMixin:
     """
     UUIDFieldMixin
 
@@ -464,10 +483,9 @@ class UUIDFieldMixin(object):
 
     DEFAULT_MAX_LENGTH = 36
 
-    def __init__(
-            self, verbose_name=None, name=None, auto=True, version=4,
-            node=None, clock_seq=None, namespace=None, uuid_name=None, *args,
-            **kwargs):
+    def __init__(self, verbose_name=None, name=None, auto=True, version=4,
+                 node=None, clock_seq=None, namespace=None, uuid_name=None, *args,
+                 **kwargs):
         if not HAS_UUID:
             raise ImproperlyConfigured("'uuid' module is required for UUIDField. (Do you have Python 2.5 or higher installed ?)")
 
@@ -485,8 +503,7 @@ class UUIDFieldMixin(object):
         self.namespace = namespace
         self.uuid_name = uuid_name or name
 
-        super(UUIDFieldMixin, self).__init__(
-            verbose_name=verbose_name, *args, **kwargs)
+        super().__init__(verbose_name=verbose_name, *args, **kwargs)
 
     def create_uuid(self):
         if not self.version or self.version == 4:
@@ -503,15 +520,15 @@ class UUIDFieldMixin(object):
             raise UUIDVersionError("UUID version %s is not valid." % self.version)
 
     def pre_save(self, model_instance, add):
-        value = super(UUIDFieldMixin, self).pre_save(model_instance, add)
+        value = super().pre_save(model_instance, add)
 
         if self.auto and add and value is None:
-            value = force_text(self.create_uuid())
+            value = force_str(self.create_uuid())
             setattr(model_instance, self.attname, value)
             return value
         else:
             if self.auto and not value:
-                value = force_text(self.create_uuid())
+                value = force_str(self.create_uuid())
                 setattr(model_instance, self.attname, value)
 
         return value
@@ -519,10 +536,10 @@ class UUIDFieldMixin(object):
     def formfield(self, **kwargs):
         if self.auto:
             return None
-        return super(UUIDFieldMixin, self).formfield(**kwargs)
+        return super().formfield(**kwargs)
 
     def deconstruct(self):
-        name, path, args, kwargs = super(UUIDFieldMixin, self).deconstruct()
+        name, path, args, kwargs = super().deconstruct()
 
         if kwargs.get('max_length', None) == self.DEFAULT_MAX_LENGTH:
             del kwargs['max_length']
@@ -554,7 +571,7 @@ class ShortUUIDField(UUIDFieldMixin, CharField):
     DEFAULT_MAX_LENGTH = 22
 
     def __init__(self, *args, **kwargs):
-        super(ShortUUIDField, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         if not HAS_SHORT_UUID:
             raise ImproperlyConfigured("'shortuuid' module is required for ShortUUIDField. (Do you have Python 2.5 or higher installed ?)")
         kwargs.setdefault('max_length', self.DEFAULT_MAX_LENGTH)
