@@ -1,8 +1,8 @@
-# coding=utf-8
-from random import choice
+# -*- coding: utf-8 -*-
+from django.core.management.base import BaseCommand
+from django.core.management.utils import get_random_secret_key
 
 from django_extensions.management.utils import signalcommand
-from django_extensions.compat import CompatibilityBaseCommand as BaseCommand
 
 
 class Command(BaseCommand):
@@ -12,4 +12,4 @@ class Command(BaseCommand):
 
     @signalcommand
     def handle(self, *args, **options):
-        return ''.join([choice('abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50)])
+        return get_random_secret_key()
