@@ -23,7 +23,7 @@ from django_extensions.management.utils import signalcommand
 USE_STATICFILES = 'django.contrib.staticfiles' in settings.INSTALLED_APPS
 
 
-class KCacheGrind(object):
+class KCacheGrind:
     def __init__(self, profiler):
         self.data = profiler.getstats()
         self.out_file = None
@@ -93,7 +93,7 @@ class Command(BaseCommand):
     args = '[optional port number, or ipaddr:port]'
 
     def add_arguments(self, parser):
-        super(Command, self).add_arguments(parser)
+        super().add_arguments(parser)
         parser.add_argument(
             'addrport', nargs='?',
             help='Optional port number, or ipaddr:port'
