@@ -6,38 +6,30 @@ import os
 import re
 import socket
 import sys
-import time
 
 import django
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.core.management.base import BaseCommand, CommandError
 from django.core.servers.basehttp import get_internal_wsgi_application
-from django.db.backends import utils
-try:
-    from django.utils.autoreload import gen_filenames
-except ImportError:  # Django >=2.2
-    from django.utils.autoreload import get_reloader
-
-    def gen_filenames():
-        return get_reloader().watched_files()
+from django.utils.autoreload import get_reloader
 
 try:
     if 'whitenoise.runserver_nostatic' in settings.INSTALLED_APPS:
         USE_STATICFILES = False
-    elif 'django.contrib.staticfiles' in settings.INSTALLED_APPS:
+    else:
         from django.contrib.staticfiles.handlers import StaticFilesHandler
         USE_STATICFILES = True
-    elif 'staticfiles' in settings.INSTALLED_APPS:
-        from staticfiles.handlers import StaticFilesHandler  # noqa
-        USE_STATICFILES = True
-    else:
-        USE_STATICFILES = False
 except ImportError:
     USE_STATICFILES = False
 
 from django_extensions.management.technical_response import null_technical_500_response
 from django_extensions.management.utils import RedirectHandler, has_ipdb, setup_logger, signalcommand
+from django_extensions.management.debug_cursor import monkey_patch_cursordebugwrapper
+
+
+def gen_filenames():
+    return get_reloader().watched_files()
 
 
 naiveip_re = re.compile(r"""^(?:
@@ -62,7 +54,7 @@ class Command(BaseCommand):
     DEFAULT_KEY_EXTENSION = ".key"
 
     def add_arguments(self, parser):
-        super(Command, self).add_arguments(parser)
+        super().add_arguments(parser)
         parser.add_argument('addrport', nargs='?',
                             help='Optional port number, or ipaddr:port')
         parser.add_argument('--ipv6', '-6', action='store_true', dest='use_ipv6', default=False,
@@ -79,6 +71,8 @@ class Command(BaseCommand):
                             help='Specifies an output file to send a copy of all messages (not flushed immediately).')
         parser.add_argument('--print-sql', action='store_true', default=False,
                             help="Print SQL queries as they're executed")
+        parser.add_argument('--print-sql-location', action='store_true', default=False,
+                            help="Show location in code where SQL query generated from")
         cert_group = parser.add_mutually_exclusive_group()
         cert_group.add_argument('--cert', dest='cert_path', action="store", type=str,
                                 help='Deprecated alias for --cert-file option.')
@@ -143,47 +137,6 @@ class Command(BaseCommand):
         werklogger.addHandler(logredirect)
         werklogger.propagate = False
 
-        if options["print_sql"]:
-            try:
-                import sqlparse
-            except ImportError:
-                sqlparse = None  # noqa
-
-            try:
-                import pygments.lexers
-                import pygments.formatters
-            except ImportError:
-                pygments = None
-
-            truncate = getattr(settings, 'RUNSERVER_PLUS_PRINT_SQL_TRUNCATE', 1000)
-
-            class PrintQueryWrapper(utils.CursorDebugWrapper):
-                def execute(self, sql, params=()):
-                    starttime = time.time()
-                    try:
-                        return utils.CursorWrapper.execute(self, sql, params)
-                    finally:
-                        execution_time = time.time() - starttime
-                        raw_sql = self.db.ops.last_executed_query(self.cursor, sql, params)
-
-                        if sqlparse:
-                            raw_sql = raw_sql[:truncate]
-                            raw_sql = sqlparse.format(raw_sql, reindent_aligned=True, truncate_strings=500)
-
-                        if pygments:
-                            raw_sql = pygments.highlight(
-                                raw_sql,
-                                pygments.lexers.get_lexer_by_name("sql"),
-                                pygments.formatters.TerminalFormatter()
-                            )
-
-                        logger.info(raw_sql)
-                        logger.info("")
-                        logger.info('[Execution time: %.6fs] [Database: %s]' % (execution_time, self.db.alias))
-                        logger.info("")
-
-            utils.CursorDebugWrapper = PrintQueryWrapper
-
         pdb_option = options['pdb']
         ipdb_option = options['ipdb']
         pm = options['pm']
@@ -259,11 +212,13 @@ class Command(BaseCommand):
             self.addr = '::1' if self.use_ipv6 else '127.0.0.1'
             self._raw_ipv6 = True
 
-        self.inner_run(options)
+        with monkey_patch_cursordebugwrapper(print_sql=options["print_sql"], print_sql_location=options["print_sql_location"], logger=logger.info, confprefix="RUNSERVER_PLUS"):
+            self.inner_run(options)
 
     def inner_run(self, options):
         try:
-            from werkzeug import run_simple, DebuggedApplication
+            from werkzeug import run_simple
+            from werkzeug.debug import DebuggedApplication
             from werkzeug.serving import WSGIRequestHandler as _WSGIRequestHandler
 
             # Set colored output
@@ -280,7 +235,7 @@ class Command(BaseCommand):
 
         class WSGIRequestHandler(_WSGIRequestHandler):
             def make_environ(self):
-                environ = super(WSGIRequestHandler, self).make_environ()
+                environ = super().make_environ()
                 if not options['keep_meta_shutdown_func']:
                     del environ['werkzeug.server.shutdown']
                 return environ
