@@ -1,18 +1,16 @@
 # -*- coding: utf-8 -*-
+import inspect
 import os
-import six
 import sys
-import time
 import traceback
 
 from django.conf import settings
 from django.core.management.base import BaseCommand, CommandError
-from django.db.backends import utils
 from django.utils.datastructures import OrderedSet
-from six import PY3
 
 from django_extensions.management.shells import import_objects
 from django_extensions.management.utils import signalcommand
+from django_extensions.management.debug_cursor import monkey_patch_cursordebugwrapper
 
 
 def use_vi_mode():
@@ -23,53 +21,48 @@ def use_vi_mode():
     return editor.startswith('vi') or editor.endswith('vim')
 
 
+def shell_runner(flags, name, help=None):
+    """
+    Decorates methods with information about the application they are starting
+
+    :param flags: The flags used to start this runner via the ArgumentParser.
+    :param name: The name of this runner for the help text for the ArgumentParser.
+    :param help: The optional help for the ArgumentParser if the dynamically generated help is not sufficient.
+    """
+
+    def decorator(fn):
+        fn.runner_flags = flags
+        fn.runner_name = name
+        fn.runner_help = help
+
+        return fn
+
+    return decorator
+
+
 class Command(BaseCommand):
     help = "Like the 'shell' command but autoloads the models of all installed Django apps."
     extra_args = None
     tests_mode = False
 
+    def __init__(self):
+        super().__init__()
+        self.runners = [member for name, member in inspect.getmembers(self)
+                        if hasattr(member, 'runner_flags')]
+
     def add_arguments(self, parser):
-        super(Command, self).add_arguments(parser)
-        parser.add_argument(
-            '--plain', action='store_true', dest='plain',
-            default=False,
-            help='Tells Django to use plain Python, not BPython nor IPython.'
-        )
-        parser.add_argument(
-            '--idle', action='store_true', dest='idle',
-            default=False,
-            help='Tells Django to use Idle.'
-        )
-        parser.add_argument(
-            '--bpython', action='store_true', dest='bpython',
-            default=False,
-            help='Tells Django to use BPython, not IPython.'
-        )
-        parser.add_argument(
-            '--ptpython', action='store_true', dest='ptpython',
-            default=False,
-            help='Tells Django to use PTPython, not IPython.'
-        )
-        parser.add_argument(
-            '--ptipython', action='store_true', dest='ptipython',
-            default=False,
-            help='Tells Django to use PT-IPython, not IPython.'
-        )
-        parser.add_argument(
-            '--ipython', action='store_true', dest='ipython',
-            default=False,
-            help='Tells Django to use IPython, not BPython.'
-        )
-        parser.add_argument(
-            '--notebook', action='store_true', dest='notebook',
-            default=False,
-            help='Tells Django to use IPython Notebook.'
-        )
-        parser.add_argument(
-            '--kernel', action='store_true', dest='kernel',
-            default=False,
-            help='Tells Django to start an IPython Kernel.'
-        )
+        super().add_arguments(parser)
+
+        group = parser.add_mutually_exclusive_group()
+        for runner in self.runners:
+            if runner.runner_help:
+                help = runner.runner_help
+            else:
+                help = 'Tells Django to use %s.' % runner.runner_name
+
+            group.add_argument(
+                *runner.runner_flags, action='store_const', dest='runner', const=runner, help=help)
+
         parser.add_argument(
             '--connection-file', action='store', dest='connection_file',
             help='Specifies the connection file to use if using the --kernel option'
@@ -89,6 +82,11 @@ class Command(BaseCommand):
             default=False,
             help="Print SQL queries as they're executed"
         )
+        parser.add_argument(
+            '--print-sql-location', action='store_true',
+            default=False,
+            help="Show location in code where SQL query generated from"
+        )
         parser.add_argument(
             '--dont-load', action='append', dest='dont_load', default=[],
             help='Ignore autoloading of some apps/models. Can be used several times.'
@@ -108,17 +106,19 @@ class Command(BaseCommand):
             dest='no_browser',
             help='Don\'t open the notebook in a browser after startup.'
         )
+        parser.add_argument(
+            '-c', '--command',
+            help='Instead of opening an interactive shell, run a command as Django and exit.',
+        )
 
     def run_from_argv(self, argv):
         if '--' in argv[2:]:
             idx = argv.index('--')
             self.extra_args = argv[idx + 1:]
             argv = argv[:idx]
-        return super(Command, self).run_from_argv(argv)
+        return super().run_from_argv(argv)
 
     def get_ipython_arguments(self, options):
-        if self.extra_args:
-            return self.extra_args
         ipython_args = 'IPYTHON_ARGUMENTS'
         arguments = getattr(settings, ipython_args, [])
         if not arguments:
@@ -126,8 +126,6 @@ class Command(BaseCommand):
         return arguments
 
     def get_notebook_arguments(self, options):
-        if self.extra_args:
-            return self.extra_args
         notebook_args = 'NOTEBOOK_ARGUMENTS'
         arguments = getattr(settings, notebook_args, [])
         if not arguments:
@@ -141,6 +139,7 @@ class Command(BaseCommand):
             self.tests_imported_objects = imported_objects
         return imported_objects
 
+    @shell_runner(flags=['--kernel'], name='IPython Kernel')
     def get_kernel(self, options):
         try:
             from IPython import release
@@ -163,92 +162,148 @@ class Command(BaseCommand):
             start_kernel(**kwargs)
         return run_kernel
 
+    def load_base_kernel_spec(self, app):
+        """Finds and returns the base Python kernelspec to extend from."""
+        ksm = app.kernel_spec_manager
+        try_spec_names = getattr(settings, 'NOTEBOOK_KERNEL_SPEC_NAMES', [
+            'python3',
+            'python',
+        ])
+
+        if isinstance(try_spec_names, str):
+            try_spec_names = [try_spec_names]
+
+        ks = None
+        for spec_name in try_spec_names:
+            try:
+                ks = ksm.get_kernel_spec(spec_name)
+                break
+            except Exception:
+                continue
+        if not ks:
+            raise CommandError("No notebook (Python) kernel specs found. Tried %r" % try_spec_names)
+
+        return ks
+
+    def generate_kernel_specs(self, app, ipython_arguments):
+        """Generate an IPython >= 3.0 kernelspec that loads django extensions"""
+        ks = self.load_base_kernel_spec(app)
+        ks.argv.extend(ipython_arguments)
+        ks.display_name = getattr(settings, 'IPYTHON_KERNEL_DISPLAY_NAME', "Django Shell-Plus")
+
+        manage_py_dir, manage_py = os.path.split(os.path.realpath(sys.argv[0]))
+        if manage_py == 'manage.py' and os.path.isdir(manage_py_dir):
+            pythonpath = ks.env.get('PYTHONPATH', os.environ.get('PYTHONPATH', ''))
+            pythonpath = pythonpath.split(os.pathsep)
+            if manage_py_dir not in pythonpath:
+                pythonpath.append(manage_py_dir)
+
+            ks.env['PYTHONPATH'] = os.pathsep.join(filter(None, pythonpath))
+
+        return {'django_extensions': ks}
+
+    def run_notebookapp(self, app, options, use_kernel_specs=True):
+        no_browser = options['no_browser']
+
+        if self.extra_args:
+            # if another '--' is found split the arguments notebook, ipython
+            if '--' in self.extra_args:
+                idx = self.extra_args.index('--')
+                notebook_arguments = self.extra_args[:idx]
+                ipython_arguments = self.extra_args[idx + 1:]
+            # otherwise pass the arguments to the notebook
+            else:
+                notebook_arguments = self.extra_args
+                ipython_arguments = []
+        else:
+            notebook_arguments = self.get_notebook_arguments(options)
+            ipython_arguments = self.get_ipython_arguments(options)
+
+        # Treat IPYTHON_ARGUMENTS from settings
+        if 'django_extensions.management.notebook_extension' not in ipython_arguments:
+            ipython_arguments.extend(['--ext', 'django_extensions.management.notebook_extension'])
+
+        # Treat NOTEBOOK_ARGUMENTS from settings
+        if no_browser and '--no-browser' not in notebook_arguments:
+            notebook_arguments.append('--no-browser')
+        if '--notebook-dir' not in notebook_arguments and not any(e.startswith('--notebook-dir=') for e in notebook_arguments):
+            notebook_arguments.extend(['--notebook-dir', '.'])
+
+        # IPython < 3 passes through kernel args from notebook CLI
+        if not use_kernel_specs:
+            notebook_arguments.extend(ipython_arguments)
+
+        app.initialize(notebook_arguments)
+
+        # IPython >= 3 uses kernelspecs to specify kernel CLI args
+        if use_kernel_specs:
+            ksm = app.kernel_spec_manager
+            for kid, ks in self.generate_kernel_specs(app, ipython_arguments).items():
+                roots = [os.path.dirname(ks.resource_dir), ksm.user_kernel_dir]
+                success = False
+                for root in roots:
+                    kernel_dir = os.path.join(root, kid)
+                    try:
+                        if not os.path.exists(kernel_dir):
+                            os.makedirs(kernel_dir)
+
+                        with open(os.path.join(kernel_dir, 'kernel.json'), 'w') as f:
+                            f.write(ks.to_json())
+
+                        success = True
+                        break
+                    except OSError:
+                        continue
+
+                if not success:
+                    raise CommandError("Could not write kernel %r in directories %r" % (kid, roots))
+
+        app.start()
+
+    @shell_runner(flags=['--notebook'], name='IPython Notebook')
     def get_notebook(self, options):
-        from IPython import release
+        try:
+            from IPython import release
+        except ImportError:
+            return traceback.format_exc()
         try:
             from notebook.notebookapp import NotebookApp
         except ImportError:
             if release.version_info[0] >= 7:
-                raise
+                return traceback.format_exc()
             try:
                 from IPython.html.notebookapp import NotebookApp
             except ImportError:
                 if release.version_info[0] >= 3:
-                    raise
+                    return traceback.format_exc()
                 try:
                     from IPython.frontend.html.notebook import notebookapp
                     NotebookApp = notebookapp.NotebookApp
                 except ImportError:
                     return traceback.format_exc()
 
-        no_browser = options['no_browser']
-
-        def install_kernel_spec(app, display_name, ipython_arguments):
-            """Install an IPython >= 3.0 kernelspec that loads django extensions"""
-            ksm = app.kernel_spec_manager
-            try_spec_names = getattr(settings, 'NOTEBOOK_KERNEL_SPEC_NAMES', [
-                'python3' if PY3 else 'python2',
-                'python',
-            ])
-            if isinstance(try_spec_names, six.string_types):
-                try_spec_names = [try_spec_names]
-            ks = None
-            for spec_name in try_spec_names:
-                try:
-                    ks = ksm.get_kernel_spec(spec_name)
-                    break
-                except Exception:
-                    continue
-            if not ks:
-                raise CommandError("No notebook (Python) kernel specs found")
-            ks.argv.extend(ipython_arguments)
-            ks.display_name = display_name
-
-            manage_py_dir, manage_py = os.path.split(os.path.realpath(sys.argv[0]))
-
-            if manage_py == 'manage.py' and os.path.isdir(manage_py_dir) and manage_py_dir != os.getcwd():
-                pythonpath = ks.env.get('PYTHONPATH', os.environ.get('PYTHONPATH', ''))
-                pythonpath = pythonpath.split(os.pathsep)
-                if manage_py_dir not in pythonpath:
-                    pythonpath.append(manage_py_dir)
-
-                ks.env['PYTHONPATH'] = os.pathsep.join(filter(None, pythonpath))
-
-            kernel_dir = os.path.join(ksm.user_kernel_dir, 'django_extensions')
-            if not os.path.exists(kernel_dir):
-                os.makedirs(kernel_dir)
-            with open(os.path.join(kernel_dir, 'kernel.json'), 'w') as f:
-                f.write(ks.to_json())
+        use_kernel_specs = release.version_info[0] >= 3
 
         def run_notebook():
             app = NotebookApp.instance()
+            self.run_notebookapp(app, options, use_kernel_specs)
 
-            # Treat IPYTHON_ARGUMENTS from settings
-            ipython_arguments = self.get_ipython_arguments(options)
-            if 'django_extensions.management.notebook_extension' not in ipython_arguments:
-                ipython_arguments.extend(['--ext', 'django_extensions.management.notebook_extension'])
-
-            # Treat NOTEBOOK_ARGUMENTS from settings
-            notebook_arguments = self.get_notebook_arguments(options)
-            if no_browser and '--no-browser' not in notebook_arguments:
-                notebook_arguments.append('--no-browser')
-            if '--notebook-dir' not in notebook_arguments and not any(e.startswith('--notebook-dir=') for e in notebook_arguments):
-                notebook_arguments.extend(['--notebook-dir', '.'])
-
-            # IPython < 3 passes through kernel args from notebook CLI
-            if release.version_info[0] < 3:
-                notebook_arguments.extend(ipython_arguments)
-
-            app.initialize(notebook_arguments)
-
-            # IPython >= 3 uses kernelspecs to specify kernel CLI args
-            if release.version_info[0] >= 3:
-                display_name = getattr(settings, 'IPYTHON_KERNEL_DISPLAY_NAME', "Django Shell-Plus")
-                install_kernel_spec(app, display_name, ipython_arguments)
-
-            app.start()
         return run_notebook
 
+    @shell_runner(flags=['--lab'], name='JupyterLab Notebook')
+    def get_jupyterlab(self, options):
+        try:
+            from jupyterlab.labapp import LabApp
+        except ImportError:
+            return traceback.format_exc()
+
+        def run_jupyterlab():
+            app = LabApp.instance()
+            self.run_notebookapp(app, options)
+
+        return run_jupyterlab
+
+    @shell_runner(flags=['--plain'], name='plain Python')
     def get_plain(self, options):
         # Using normal Python shell
         import code
@@ -297,6 +352,7 @@ class Command(BaseCommand):
             code.interact(local=imported_objects)
         return run_plain
 
+    @shell_runner(flags=['--bpython'], name='BPython')
     def get_bpython(self, options):
         try:
             from bpython import embed
@@ -311,13 +367,14 @@ class Command(BaseCommand):
             embed(imported_objects, **kwargs)
         return run_bpython
 
+    @shell_runner(flags=['--ipython'], name='IPython')
     def get_ipython(self, options):
         try:
             from IPython import start_ipython
 
             def run_ipython():
                 imported_objects = self.get_imported_objects(options)
-                ipython_arguments = self.get_ipython_arguments(options)
+                ipython_arguments = self.extra_args or self.get_ipython_arguments(options)
                 start_ipython(argv=ipython_arguments, user_ns=imported_objects)
             return run_ipython
         except ImportError:
@@ -337,6 +394,7 @@ class Command(BaseCommand):
                 shell.mainloop()
             return run_ipython
 
+    @shell_runner(flags=['--ptpython'], name='PTPython')
     def get_ptpython(self, options):
         try:
             from ptpython.repl import embed, run_config
@@ -354,6 +412,7 @@ class Command(BaseCommand):
                   vi_mode=options['vi_mode'], configure=run_config)
         return run_ptpython
 
+    @shell_runner(flags=['--ptipython'], name='PT-IPython')
     def get_ptipython(self, options):
         try:
             from ptpython.repl import run_config
@@ -373,6 +432,7 @@ class Command(BaseCommand):
                   vi_mode=options['vi_mode'], configure=run_config)
         return run_ptipython
 
+    @shell_runner(flags=['--idle'], name='Idle')
     def get_idle(self, options):
         from idlelib.pyshell import main
 
@@ -426,125 +486,79 @@ for k, m in shells.import_objects({}, no_style()).items():
 
     @signalcommand
     def handle(self, *args, **options):
-        use_kernel = options['kernel']
-        use_notebook = options['notebook']
-        use_ipython = options['ipython']
-        use_bpython = options['bpython']
-        use_plain = options['plain']
-        use_idle = options['idle']
-        use_ptpython = options['ptpython']
-        use_ptipython = options['ptipython']
         verbosity = options["verbosity"]
+        get_runner = options['runner']
         print_sql = getattr(settings, 'SHELL_PLUS_PRINT_SQL', False)
-        truncate = getattr(settings, 'SHELL_PLUS_PRINT_SQL_TRUNCATE', 1000)
+        runner = None
+        runner_name = None
 
-        if options["print_sql"] or print_sql:
+        with monkey_patch_cursordebugwrapper(print_sql=options["print_sql"] or print_sql, print_sql_location=options["print_sql_location"], confprefix="SHELL_PLUS"):
+            SETTINGS_SHELL_PLUS = getattr(settings, 'SHELL_PLUS', None)
 
-            # Code from http://gist.github.com/118990
-            try:
-                import sqlparse
+            def get_runner_by_flag(flag):
+                for runner in self.runners:
+                    if flag in runner.runner_flags:
+                        return runner
+                return None
 
-                sqlparse_format_kwargs_defaults = dict(
-                    reindent_aligned=True,
-                    truncate_strings=500,
-                )
-                sqlparse_format_kwargs = getattr(settings, 'SHELL_PLUS_SQLPARSE_FORMAT_KWARGS', sqlparse_format_kwargs_defaults)
-            except ImportError:
-                sqlparse = None
+            self.set_application_name(options)
 
-            try:
-                import pygments.lexers
-                import pygments.formatters
+            if not get_runner and SETTINGS_SHELL_PLUS:
+                get_runner = get_runner_by_flag('--%s' % SETTINGS_SHELL_PLUS)
+                if not get_runner:
+                    runner = None
+                    runner_name = SETTINGS_SHELL_PLUS
 
-                pygments_formatter = getattr(settings, 'SHELL_PLUS_PYGMENTS_FORMATTER', pygments.formatters.TerminalFormatter)
-                pygments_formatter_kwargs = getattr(settings, 'SHELL_PLUS_PYGMENTS_FORMATTER_KWARGS', {})
-            except ImportError:
-                pygments = None
+            if get_runner:
+                runner = get_runner(options)
+                runner_name = get_runner.runner_name
+            else:
+                def try_runner(get_runner):
+                    runner_name = get_runner.runner_name
+                    if verbosity > 2:
+                        print(self.style.NOTICE("Trying: %s" % runner_name))
 
-            class PrintQueryWrapper(utils.CursorDebugWrapper):
-                def execute(self, sql, params=()):
-                    starttime = time.time()
-                    try:
-                        return utils.CursorWrapper.execute(self, sql, params)
-                    finally:
-                        execution_time = time.time() - starttime
-                        raw_sql = self.db.ops.last_executed_query(self.cursor, sql, params)
+                    runner = get_runner(options)
+                    if callable(runner):
+                        if verbosity > 1:
+                            print(self.style.NOTICE("Using: %s" % runner_name))
+                        return runner
+                    return None
 
-                        if sqlparse:
-                            raw_sql = raw_sql[:truncate]
-                            raw_sql = sqlparse.format(raw_sql, **sqlparse_format_kwargs)
+                tried_runners = set()
 
-                        if pygments:
-                            raw_sql = pygments.highlight(
-                                raw_sql,
-                                pygments.lexers.get_lexer_by_name("sql"),
-                                pygments_formatter(**pygments_formatter_kwargs),
-                            )
+                # try the runners that are least unexpected (normal shell runners)
+                preferred_runners = ['ptipython', 'ptpython', 'bpython', 'ipython', 'plain']
+                for flag_suffix in preferred_runners:
+                    get_runner = get_runner_by_flag('--%s' % flag_suffix)
+                    tried_runners.add(get_runner)
+                    runner = try_runner(get_runner)
+                    if runner:
+                        runner_name = get_runner.runner_name
+                        break
 
-                        print(raw_sql)
-                        print("")
-                        print('Execution time: %.6fs [Database: %s]' % (execution_time, self.db.alias))
-                        print("")
+                # try any remaining runners if needed
+                if not runner:
+                    for get_runner in self.runners:
+                        if get_runner not in tried_runners:
+                            runner = try_runner(get_runner)
+                            if runner:
+                                runner_name = get_runner.runner_name
+                                break
 
-            utils.CursorDebugWrapper = PrintQueryWrapper
+            if not callable(runner):
+                if runner:
+                    print(runner)
+                if not runner_name:
+                    raise CommandError("No shell runner could be found.")
+                raise CommandError("Could not load shell runner: '%s'." % runner_name)
 
-        shells = (
-            ('ptipython', self.get_ptipython),
-            ('ptpython', self.get_ptpython),
-            ('bpython', self.get_bpython),
-            ('ipython', self.get_ipython),
-            ('plain', self.get_plain),
-            ('idle', self.get_idle),
-        )
-        SETTINGS_SHELL_PLUS = getattr(settings, 'SHELL_PLUS', None)
+            if self.tests_mode:
+                return 130
 
-        shell = None
-        shell_name = "any"
-        self.set_application_name(options)
-        if use_kernel:
-            shell = self.get_kernel(options)
-            shell_name = "IPython Kernel"
-        elif use_notebook:
-            shell = self.get_notebook(options)
-            shell_name = "IPython Notebook"
-        elif use_plain:
-            shell = self.get_plain(options)
-            shell_name = "plain"
-        elif use_ipython:
-            shell = self.get_ipython(options)
-            shell_name = "IPython"
-        elif use_bpython:
-            shell = self.get_bpython(options)
-            shell_name = "BPython"
-        elif use_ptpython:
-            shell = self.get_ptpython(options)
-            shell_name = "ptpython"
-        elif use_ptipython:
-            shell = self.get_ptipython(options)
-            shell_name = "ptipython"
-        elif use_idle:
-            shell = self.get_idle(options)
-            shell_name = "idle"
-        elif SETTINGS_SHELL_PLUS:
-            shell_name = SETTINGS_SHELL_PLUS
-            shell = dict(shells)[shell_name](options)
-        else:
-            for shell_name, func in shells:
-                if verbosity > 2:
-                    print(self.style.NOTICE("Trying shell: %s" % shell_name))
-                shell = func(options)
-                if callable(shell):
-                    if verbosity > 1:
-                        print(self.style.NOTICE("Using shell: %s" % shell_name))
-                    break
+            if options['command']:
+                imported_objects = self.get_imported_objects(options)
+                exec(options['command'], {}, imported_objects)
+                return
 
-        if not callable(shell):
-            if shell:
-                print(shell)
-            print(self.style.ERROR("Could not load %s interactive Python environment." % shell_name))
-            return
-
-        if self.tests_mode:
-            return 130
-
-        shell()
+            runner()
