@@ -69,7 +69,7 @@ def all_local_fields(meta):
     return all_fields
 
 
-class SQLDiff(object):
+class SQLDiff:
     DATA_TYPES_REVERSE_OVERRIDE = {}  # type: Dict[int, Union[str, Callable]]
 
     IGNORE_MISSING_TABLES = [
@@ -203,10 +203,6 @@ class SQLDiff(object):
 
         self.introspection = connection.introspection
 
-        self.cursor = connection.cursor()
-        self.django_tables = self.introspection.django_table_names(only_existing=options['only_existing'])
-        # TODO: We are losing information about tables which are views here
-        self.db_tables = [table_info.name for table_info in self.introspection.get_table_list(self.cursor)]
         self.differences = []
         self.unknown_db_fields = {}
         self.new_db_fields = set()
@@ -231,6 +227,12 @@ class SQLDiff(object):
             'notnull-differ': self.SQL_NOTNULL_DIFFER,
         }
 
+    def load(self):
+        self.cursor = connection.cursor()
+        self.django_tables = self.introspection.django_table_names(only_existing=self.options['only_existing'])
+        # TODO: We are losing information about tables which are views here
+        self.db_tables = [table_info.name for table_info in self.introspection.get_table_list(self.cursor)]
+
         if self.can_detect_notnull_differ:
             self.load_null()
 
@@ -254,14 +256,21 @@ class SQLDiff(object):
         # type: () -> Dict[int, Union[str, Callable]]
         return self.DATA_TYPES_REVERSE_OVERRIDE
 
+    def format_field_names(self, field_names):
+        return field_names
+
     def sql_to_dict(self, query, param):
-        """ sql_to_dict(query, param) -> list of dicts
+        """
+        Execute query and return a dict
+
+        sql_to_dict(query, param) -> list of dicts
 
         code from snippet at http://www.djangosnippets.org/snippets/1383/
         """
         cursor = connection.cursor()
         cursor.execute(query, param)
         fieldnames = [name[0] for name in cursor.description]
+        fieldnames = self.format_field_names(fieldnames)
         result = []
         for row in cursor.fetchall():
             rowset = []
@@ -345,7 +354,7 @@ class SQLDiff(object):
         tablespace = field.db_tablespace
         if not tablespace:
             tablespace = "public"
-        if (tablespace, table_name, field.column) in self.unsigned:
+        if (tablespace, table_name, field.column) in self.unsigned and self.unsigned_suffix not in field_db_type:
             field_db_type = '%s %s' % (field_db_type, self.unsigned_suffix)
 
         return field_db_type
@@ -358,8 +367,8 @@ class SQLDiff(object):
             module_path, package_name = class_path.rsplit('.', 1)
             module = importlib.import_module(module_path)
             return getattr(module, package_name)
-        else:
-            return getattr(models, class_path)
+
+        return getattr(models, class_path)
 
     def get_field_db_nullable(self, field, table_name):
         tablespace = field.db_tablespace
@@ -430,8 +439,10 @@ class SQLDiff(object):
 
             columns = constraint['columns']
             if len(columns) == 1:
-                field = fields[columns[0]]
-                if field.unique:
+                field = fields.get(columns[0])
+                if field is None:
+                    pass
+                elif field.unique:
                     continue
             else:
                 if tuple(columns) in unique_together:
@@ -478,19 +489,20 @@ class SQLDiff(object):
                 continue
 
             columns = constraint['columns']
-            if constraint['unique'] and constraint['index']:
+            field = fields.get(columns[0])
+            if (constraint['unique'] and constraint['index']) or field is None:
                 # unique indexes do not exist in django ? only unique constraints
                 pass
             elif len(columns) == 1:
-                field = fields[columns[0]]
                 if constraint['primary_key'] and field.primary_key:
                     continue
                 if constraint['foreign_key'] and isinstance(field, models.ForeignKey) and field.db_constraint:
                     continue
                 if constraint['unique'] and field.unique:
                     continue
-                if constraint['index'] and constraint['type'] == 'idx' and constraint['orders'] and field.unique:
+                if constraint['index'] and constraint['type'] == 'idx' and constraint.get('orders') and field.unique:
                     # django automatically creates a _like varchar_pattern_ops/text_pattern_ops index see https://code.djangoproject.com/ticket/12234
+                    # note: mysql does not have and/or introspect and fill the 'orders' attribute of constraint information
                     continue
                 if constraint['index'] and field.db_index:
                     continue
@@ -670,7 +682,7 @@ class SQLDiff(object):
         self.has_differences = max([len(diffs) for _app_label, _model_name, diffs in self.differences])
 
     def print_diff(self, style=no_style()):
-        """ print differences to stdout """
+        """ Print differences to stdout """
         if self.options['sql']:
             self.print_diff_sql(style)
         else:
@@ -740,6 +752,13 @@ class SQLDiff(object):
 
 class GenericSQLDiff(SQLDiff):
     can_detect_notnull_differ = False
+    can_detect_unsigned_differ = False
+
+    def load_null(self):
+        pass
+
+    def load_unsigned(self):
+        pass
 
 
 class MySQLDiff(SQLDiff):
@@ -747,11 +766,14 @@ class MySQLDiff(SQLDiff):
     can_detect_unsigned_differ = True
     unsigned_suffix = 'UNSIGNED'
 
-    def __init__(self, *args, **kwargs):
-        super(MySQLDiff, self).__init__(*args, **kwargs)
+    def load(self):
+        super().load()
         self.auto_increment = set()
         self.load_auto_increment()
 
+    def format_field_names(self, field_names):
+        return [f.lower() for f in field_names]
+
     def load_null(self):
         tablespace = 'public'
         for table_name in self.db_tables:
@@ -793,27 +815,111 @@ class MySQLDiff(SQLDiff):
     # Fixing one bug in MySQL creates another issue. So just keep in mind
     # that this is way unreliable for MySQL atm.
     def get_field_db_type(self, description, field=None, table_name=None):
-        from MySQLdb.constants import FIELD_TYPE
-        db_type = super(MySQLDiff, self).get_field_db_type(description, field, table_name)
+        db_type = super().get_field_db_type(description, field, table_name)
         if not db_type:
             return
         if field:
             # MySQL isn't really sure about char's and varchar's like sqlite
             field_type = self.get_field_model_type(field)
+
             # Fix char/varchar inconsistencies
             if self.strip_parameters(field_type) == 'char' and self.strip_parameters(db_type) == 'varchar':
                 db_type = db_type.lstrip("var")
-            # They like to call 'bool's 'tinyint(1)' and introspection makes that a integer
-            # just convert it back to it's proper type, a bool is a bool and nothing else.
-            if db_type == 'integer' and description[1] == FIELD_TYPE.TINY and description[2] == 1:
-                db_type = 'bool'
-            if (table_name, field.column) in self.auto_increment:
+
+            # They like to call bools various integer types and introspection makes that a integer
+            # just convert them all to bools
+            if self.strip_parameters(field_type) == 'bool':
+                if db_type == 'integer':
+                    db_type = 'bool'
+
+            if (table_name, field.column) in self.auto_increment and 'AUTO_INCREMENT' not in db_type:
                 db_type += ' AUTO_INCREMENT'
         return db_type
 
+    def find_index_missing_in_model(self, meta, table_indexes, table_constraints, table_name):
+        fields = dict([(field.column, field) for field in all_local_fields(meta)])
+        meta_index_names = [idx.name for idx in meta.indexes]
+        index_together = self.expand_together(meta.index_together, meta)
+        unique_together = self.expand_together(meta.unique_together, meta)
+
+        for constraint_name, constraint in six.iteritems(table_constraints):
+            if constraint_name in meta_index_names:
+                continue
+            if constraint['unique'] and not constraint['index']:
+                # unique constraints are handled by find_unique_missing_in_model
+                continue
+
+            columns = constraint['columns']
+            field = fields.get(columns[0])
+
+            # extra check removed from superclass here, otherwise function is the same
+            if len(columns) == 1:
+                if constraint['primary_key'] and field.primary_key:
+                    continue
+                if constraint['foreign_key'] and isinstance(field, models.ForeignKey) and field.db_constraint:
+                    continue
+                if constraint['unique'] and field.unique:
+                    continue
+                if constraint['index'] and constraint['type'] == 'idx' and constraint.get('orders') and field.unique:
+                    # django automatically creates a _like varchar_pattern_ops/text_pattern_ops index see https://code.djangoproject.com/ticket/12234
+                    # note: mysql does not have and/or introspect and fill the 'orders' attribute of constraint information
+                    continue
+                if constraint['index'] and field.db_index:
+                    continue
+                if constraint['check'] and field.db_check(connection=connection):
+                    continue
+                if getattr(field, 'spatial_index', False):
+                    continue
+            else:
+                if constraint['index'] and tuple(columns) in index_together:
+                    continue
+                if constraint['index'] and constraint['unique'] and tuple(columns) in unique_together:
+                    continue
+
+            self.add_difference('index-missing-in-model', table_name, constraint_name)
+
+    def find_unique_missing_in_db(self, meta, table_indexes, table_constraints, table_name, skip_list=None):
+
+        schema_editor = connection.SchemaEditorClass(connection)
+        for field in all_local_fields(meta):
+            if skip_list and field.attname in skip_list:
+                continue
+            if field.unique and meta.managed:
+                attname = field.db_column or field.attname
+                db_field_unique = table_indexes.get(attname, {}).get('unique')
+                if not db_field_unique and table_constraints:
+                    db_field_unique = any(constraint['unique'] for contraint_name, constraint in six.iteritems(table_constraints) if [attname] == constraint['columns'])
+                if attname in table_indexes and db_field_unique:
+                    continue
+
+                index_name = schema_editor._create_index_name(table_name, [attname])
+
+                self.add_difference('unique-missing-in-db', table_name, [attname], index_name + "_uniq")
+                db_type = field.db_type(connection=connection)
+                if db_type.startswith('varchar'):
+                    self.add_difference('index-missing-in-db', table_name, [attname], index_name + '_like', ' varchar_pattern_ops')
+                if db_type.startswith('text'):
+                    self.add_difference('index-missing-in-db', table_name, [attname], index_name + '_like', ' text_pattern_ops')
+
+        unique_together = self.expand_together(meta.unique_together, meta)
+
+        # This comparison changed from superclass - otherwise function is the same
+        db_unique_columns = normalize_together([v['columns'] for v in six.itervalues(table_constraints) if v['unique']])
+
+        for unique_columns in unique_together:
+            if unique_columns in db_unique_columns:
+                continue
+
+            if skip_list and unique_columns in skip_list:
+                continue
+
+            index_name = schema_editor._create_index_name(table_name, unique_columns)
+            self.add_difference('unique-missing-in-db', table_name, unique_columns, index_name + "_uniq")
+
 
 class SqliteSQLDiff(SQLDiff):
     can_detect_notnull_differ = True
+    can_detect_unsigned_differ = False
 
     def load_null(self):
         for table_name in self.db_tables:
@@ -825,11 +931,15 @@ class SqliteSQLDiff(SQLDiff):
                 key = (tablespace, table_name, table_info['name'])
                 self.null[key] = not table_info['notnull']
 
+    def load_unsigned(self):
+        pass
+
     # Unique does not seem to be implied on Sqlite for Primary_key's
     # if this is more generic among databases this might be usefull
     # to add to the superclass's find_unique_missing_in_db method
-    def find_unique_missing_in_db(self, meta, table_indexes, table_constraints, table_name):
-        skip_list = []
+    def find_unique_missing_in_db(self, meta, table_indexes, table_constraints, table_name, skip_list=None):
+        if skip_list is None:
+            skip_list = []
 
         unique_columns = [field.db_column or field.attname for field in all_local_fields(meta) if field.unique]
 
@@ -847,7 +957,7 @@ class SqliteSQLDiff(SQLDiff):
             if unique_columns in db_unique_columns:
                 skip_list.append(unique_columns)
 
-        super(SqliteSQLDiff, self).find_unique_missing_in_db(meta, table_indexes, table_constraints, table_name, skip_list=skip_list)
+        super().find_unique_missing_in_db(meta, table_indexes, table_constraints, table_name, skip_list=skip_list)
 
     # Finding Indexes by using the get_indexes dictionary doesn't seem to work
     # for sqlite.
@@ -858,9 +968,9 @@ class SqliteSQLDiff(SQLDiff):
         pass
 
     def get_field_db_type(self, description, field=None, table_name=None):
-        db_type = super(SqliteSQLDiff, self).get_field_db_type(description, field, table_name)
+        db_type = super().get_field_db_type(description, field, table_name)
         if not db_type:
-            return
+            return None
         if field:
             field_type = self.get_field_model_type(field)
             # Fix char/varchar inconsistencies
@@ -874,32 +984,33 @@ class PostgresqlSQLDiff(SQLDiff):
     can_detect_unsigned_differ = True
 
     DATA_TYPES_REVERSE_NAME = {
-        'hstore': 'django_hstore.hstore.DictionaryField',
+        'hstore': 'django.contrib.postgres.fields.HStoreField',
+        'jsonb': 'django.contrib.postgres.fields.JSONField',
     }
 
     # Hopefully in the future we can add constraint checking and other more
     # advanced checks based on this database.
     SQL_LOAD_CONSTRAINTS = """
-    SELECT nspname, relname, conname, attname, pg_get_constraintdef(pg_constraint.oid)
-    FROM pg_constraint
-    INNER JOIN pg_attribute ON pg_constraint.conrelid = pg_attribute.attrelid AND pg_attribute.attnum = any(pg_constraint.conkey)
-    INNER JOIN pg_class ON conrelid=pg_class.oid
-    INNER JOIN pg_namespace ON pg_namespace.oid=pg_class.relnamespace
-    ORDER BY CASE WHEN contype='f' THEN 0 ELSE 1 END,contype,nspname,relname,conname;
+        SELECT nspname, relname, conname, attname, pg_get_constraintdef(pg_constraint.oid)
+        FROM pg_constraint
+        INNER JOIN pg_attribute ON pg_constraint.conrelid = pg_attribute.attrelid AND pg_attribute.attnum = any(pg_constraint.conkey)
+        INNER JOIN pg_class ON conrelid=pg_class.oid
+        INNER JOIN pg_namespace ON pg_namespace.oid=pg_class.relnamespace
+        ORDER BY CASE WHEN contype='f' THEN 0 ELSE 1 END,contype,nspname,relname,conname;
     """
     SQL_LOAD_NULL = """
-    SELECT nspname, relname, attname, attnotnull
-    FROM pg_attribute
-    INNER JOIN pg_class ON attrelid=pg_class.oid
-    INNER JOIN pg_namespace ON pg_namespace.oid=pg_class.relnamespace;
+        SELECT nspname, relname, attname, attnotnull
+        FROM pg_attribute
+        INNER JOIN pg_class ON attrelid=pg_class.oid
+        INNER JOIN pg_namespace ON pg_namespace.oid=pg_class.relnamespace;
     """
 
     SQL_FIELD_TYPE_DIFFER = lambda self, style, qn, args: "%s %s\n\t%s %s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD('ALTER'), style.SQL_FIELD(qn(args[1])), style.SQL_KEYWORD("TYPE"), style.SQL_COLTYPE(args[2]))
     SQL_FIELD_PARAMETER_DIFFER = lambda self, style, qn, args: "%s %s\n\t%s %s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD('ALTER'), style.SQL_FIELD(qn(args[1])), style.SQL_KEYWORD("TYPE"), style.SQL_COLTYPE(args[2]))
     SQL_NOTNULL_DIFFER = lambda self, style, qn, args: "%s %s\n\t%s %s %s %s;" % (style.SQL_KEYWORD('ALTER TABLE'), style.SQL_TABLE(qn(args[0])), style.SQL_KEYWORD('ALTER COLUMN'), style.SQL_FIELD(qn(args[1])), style.SQL_KEYWORD(args[2]), style.SQL_KEYWORD('NOT NULL'))
 
-    def __init__(self, *args, **kwargs):
-        super(PostgresqlSQLDiff, self).__init__(*args, **kwargs)
+    def load(self):
+        super().load()
         self.check_constraints = {}
         self.load_constraints()
 
@@ -954,7 +1065,11 @@ class PostgresqlSQLDiff(SQLDiff):
         }
 
     def get_constraints(self, cursor, table_name, introspection):
-        """ backport of django's introspection.get_constraints(...) """
+        """
+        Find constraints for table
+
+        Backport of django's introspection.get_constraints(...)
+        """
         constraints = {}
         # Loop over the key table, collecting things as constraints
         # This will get PKs, FKs, and uniques, but not CHECK
@@ -1065,7 +1180,7 @@ class PostgresqlSQLDiff(SQLDiff):
     #     return kwargs
 
     def get_field_db_type(self, description, field=None, table_name=None):
-        db_type = super(PostgresqlSQLDiff, self).get_field_db_type(description, field, table_name)
+        db_type = super().get_field_db_type(description, field, table_name)
         if not db_type:
             return
         if field:
@@ -1125,7 +1240,7 @@ class PostgresqlSQLDiff(SQLDiff):
             if field.primary_key and db_type=='integer':
                 db_type = 'serial'
             return model_type, db_type
-        super(PostgresqlSQLDiff, self).find_field_type_differ(meta, table_description, table_name, callback)
+        super().find_field_type_differ(meta, table_description, table_name, callback)
     """
 
 
@@ -1150,7 +1265,7 @@ to check/debug ur models compared to the real database tables and columns."""
     output_transaction = False
 
     def add_arguments(self, parser):
-        super(Command, self).add_arguments(parser)
+        super().add_arguments(parser)
         parser.add_argument('app_label', nargs='*')
         parser.add_argument(
             '--all-applications', '-a', action='store_true',
@@ -1191,7 +1306,7 @@ to check/debug ur models compared to the real database tables and columns."""
         )
 
     def __init__(self, *args, **kwargs):
-        super(Command, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.exit_code = 1
 
     @signalcommand
@@ -1242,6 +1357,7 @@ Edit your settings file and change DATABASE_ENGINE to something like 'postgresql
 
         cls = DATABASE_SQLDIFF_CLASSES.get(engine, GenericSQLDiff)
         sqldiff_instance = cls(app_models, options, stdout=self.stdout, stderr=self.stderr)
+        sqldiff_instance.load()
         sqldiff_instance.find_differences()
         if not sqldiff_instance.has_differences:
             self.exit_code = 0
@@ -1249,7 +1365,7 @@ Edit your settings file and change DATABASE_ENGINE to something like 'postgresql
 
     def execute(self, *args, **options):
         try:
-            super(Command, self).execute(*args, **options)
+            super().execute(*args, **options)
         except CommandError as e:
             if options['traceback']:
                 raise
@@ -1262,5 +1378,5 @@ Edit your settings file and change DATABASE_ENGINE to something like 'postgresql
             sys.exit(2)
 
     def run_from_argv(self, argv):
-        super(Command, self).run_from_argv(argv)
+        super().run_from_argv(argv)
         sys.exit(self.exit_code)
