@@ -1,16 +1,15 @@
 # -*- coding: utf-8 -*-
 """
-sqldns.py - Prints Data Source Name on stdout
+sqldns.py
 
+Prints Data Source Name on stdout
 """
 
 import sys
 from django.conf import settings
-from django.core.management.base import CommandError
+from django.core.management.base import BaseCommand, CommandError
 from django.core.management.color import color_style
 
-from django_extensions.compat import CompatibilityBaseCommand as BaseCommand
-
 
 class Command(BaseCommand):
     help = """Prints DSN on stdout, as specified in settings.py
@@ -21,27 +20,36 @@ class Command(BaseCommand):
     can_import_settings = True
 
     def add_arguments(self, parser):
-        parser.add_argument('-R', '--router', action='store',
-                    dest='router', default='default',
-                    help='Use this router-database other then default')
-        parser.add_argument('-s', '--style', action='store',
-                    dest='style', default=None,
-                    help='DSN format style: keyvalue, uri, pgpass, all')
-        parser.add_argument('-a', '--all', action='store_true',
-                    dest='all', default=False,
-                    help='Show DSN for all database routes')
-        parser.add_argument('-q', '--quiet', action='store_true',
-                    dest='quiet', default=False,
-                    help='Quiet mode only show DSN')
+        super().add_arguments(parser)
+        parser.add_argument(
+            '-R', '--router', action='store',
+            dest='router', default='default',
+            help='Use this router-database other then default'
+        )
+        parser.add_argument(
+            '-s', '--style', action='store',
+            dest='style', default=None,
+            help='DSN format style: keyvalue, uri, pgpass, all'
+        )
+        parser.add_argument(
+            '-a', '--all', action='store_true',
+            dest='all', default=False,
+            help='Show DSN for all database routes'
+        )
+        parser.add_argument(
+            '-q', '--quiet', action='store_true',
+            dest='quiet', default=False,
+            help='Quiet mode only show DSN'
+        )
 
     def handle(self, *args, **options):
         self.style = color_style()
-        all_routers = options.get('all')
+        all_routers = options['all']
 
         if all_routers:
             routers = settings.DATABASES.keys()
         else:
-            routers = [options.get('router')]
+            routers = [options['router']]
 
         for i, router in enumerate(routers):
             if i != 0:
@@ -50,8 +58,8 @@ class Command(BaseCommand):
 
     def show_dsn(self, router, options):
         dbinfo = settings.DATABASES.get(router)
-        quiet = options.get('quiet')
-        dsn_style = options.get('style')
+        quiet = options['quiet']
+        dsn_style = options['style']
 
         if dbinfo is None:
             raise CommandError("Unknown database router %s" % router)
@@ -66,22 +74,12 @@ class Command(BaseCommand):
         dsn = []
 
         if engine == 'mysql':
-            dsnstr = 'host="{0}", db="{2}", user="{3}", passwd="{4}"'
-            if dbport is not None:
-                dsnstr += ', port="{1}"'
-
-            dsn.append(dsnstr.format(dbhost,
-                                     dbport,
-                                     dbname,
-                                     dbuser,
-                                     dbpass))
-
-        elif engine in ['postgresql_psycopg2', 'postgis']:
-            dsn = self.postgresql(dbhost, dbport, dbname, dbuser, dbpass, dsn_style=dsn_style)
-
+            dsn.append(self._mysql(dbhost, dbport, dbname, dbuser, dbpass))
+        elif engine in ['postgresql', 'postgresql_psycopg2', 'postgis']:
+            dsn.extend(self._postgresql(
+                dbhost, dbport, dbname, dbuser, dbpass, dsn_style=dsn_style))
         elif engine == 'sqlite3':
             dsn.append('{}'.format(dbname))
-
         else:
             dsn.append(self.style.ERROR('Unknown database, can''t generate DSN'))
 
@@ -91,8 +89,16 @@ class Command(BaseCommand):
         for output in dsn:
             sys.stdout.write("{}\n".format(output))
 
-    def postgresql(self, dbhost, dbport, dbname, dbuser, dbpass, dsn_style=None):
-        """PostgreSQL psycopg2 driver  accepts two syntaxes
+    def _mysql(self, dbhost, dbport, dbname, dbuser, dbpass):
+        dsnstr = 'host="{0}", db="{2}", user="{3}", passwd="{4}"'
+
+        if dbport is not None:
+            dsnstr += ', port="{1}"'
+
+        return dsnstr.format(dbhost, dbport, dbname, dbuser, dbpass)
+
+    def _postgresql(self, dbhost, dbport, dbname, dbuser, dbpass, dsn_style=None):  # noqa
+        """PostgreSQL psycopg2 driver accepts two syntaxes
 
         Plus a string for .pgpass file
         """
@@ -122,23 +128,14 @@ class Command(BaseCommand):
                                      dbpass))
 
         if dsn_style == 'all' or dsn_style == 'uri':
-            if dbport is not None:
-                dsnstr = "postgresql://{3}:{4}@{0}:{1}/{2}"
-            else:
-                dsnstr = "postgresql://{3}:{4}@{0}/{2}"
+            dsnstr = "postgresql://{user}:{password}@{host}/{name}"
 
-            dsn.append(dsnstr.format(dbhost,
-                                     dbport,
-                                     dbname,
-                                     dbuser,
-                                     dbpass,))
+            dsn.append(dsnstr.format(
+                host="{host}:{port}".format(host=dbhost, port=dbport) if dbport else dbhost,  # noqa
+                name=dbname, user=dbuser, password=dbpass))
 
         if dsn_style == 'all' or dsn_style == 'pgpass':
-            if dbport is not None:
-                dbport = 5432
-            dsn.append('{0}:{1}:{2}:{3}:{4}'.format(dbhost,
-                                                    dbport,
-                                                    dbname,
-                                                    dbuser,
-                                                    dbpass))
+            dsn.append(':'.join(map(str, filter(
+                None, [dbhost, dbport, dbname, dbuser, dbpass]))))
+
         return dsn
