@@ -1,4 +1,4 @@
-# coding=utf-8
+# -*- coding: utf-8 -*-
 """
 Similar to syntax_color.py but this is intended more for being able to
 copy+paste actual code into your Django templates without needing to
@@ -27,10 +27,8 @@ Example:
  def need_food(self):
      print("Love is <colder> than &death&")
  {% endhighlight %}
-
 """
 
-import django
 from django import template
 from django.template import (
     Context, Node, Template, TemplateSyntaxError, Variable,
@@ -43,22 +41,16 @@ try:
     from pygments.lexers import get_lexer_by_name
     from pygments.formatters import HtmlFormatter
     HAS_PYGMENTS = True
-except ImportError:
+except ImportError:  # pragma: no cover
     HAS_PYGMENTS = False
 
 register = template.Library()
 
 
+@register.filter(is_safe=True)
 @stringfilter
 def parse_template(value):
     return mark_safe(Template(value).render(Context()))
-parse_template.is_safe = True
-
-if django.get_version() >= "1.4":
-    register.filter(parse_template, is_safe=True)
-else:
-    parse_template.is_safe = True
-    register.filter(parse_template)
 
 
 class CodeNode(Node):
@@ -84,7 +76,7 @@ class CodeNode(Node):
 @register.tag
 def highlight(parser, token):
     """
-    Allows you to put a highlighted source code <pre> block in your code.
+    Tag to put a highlighted source code <pre> block in your code.
     This takes two arguments, the language and a little explaination message
     that will be generated before the code.  The second argument is optional.
 
@@ -100,7 +92,7 @@ def highlight(parser, token):
       {% endhighlight %}
 
     """
-    if not HAS_PYGMENTS:
+    if not HAS_PYGMENTS:  # pragma: no cover
         raise ImportError("Please install 'pygments' library to use highlighting.")
     nodelist = parser.parse(('endhighlight',))
     parser.delete_first_token()
