@@ -1,10 +1,12 @@
 # -*- coding: utf-8 -*-
-import mock
-from django_extensions.management.commands import mail_debug
+from django.core.management import call_command
+try:
+    from unittest import mock
+except ImportError:
+    import mock
 
 
 def test_initialize_mail_server():
     with mock.patch('django_extensions.management.commands.mail_debug.asyncore.loop') as loop:
-        command = mail_debug.Command()
-        command.run_from_argv(['manage.py', 'mail_debug'])
+        call_command('mail_debug')
         assert loop.called, 'asyncore.loop was not called'
