@@ -4,7 +4,10 @@ import six
 import mock
 import logging
 import importlib
+import pytest
+import sys
 
+from django.conf import settings
 from django.core.management import call_command, find_commands, load_command_class
 from django.test import TestCase
 from six import StringIO
@@ -138,6 +141,7 @@ class GraphModelsTests(TestCase):
             '*WildcardPrefixExclude',
             'WildcardSuffixExclude*',
             '*WildcardBothExclude*',
+            '*Include',
         ]
         # Any model name should be used if neither include or exclude
         # are defined.
@@ -220,6 +224,19 @@ class GraphModelsTests(TestCase):
             None,
             exclude_models,
         ))
+        # Test with `exclude_models` and `include_models` combined
+        # where the user wants to exclude some models through a wildcard
+        # while still being able to include given models
+        self.assertTrue(use_model(
+            'MyWildcardPrefixInclude',
+            include_models,
+            exclude_models
+        ))
+        self.assertFalse(use_model(
+            'MyInclude',
+            include_models,
+            exclude_models
+        ))
 
     def test_no_models_dot_py(self):
         data = generate_graph_data(['testapp_with_no_models_file'])
@@ -248,6 +265,69 @@ class ShowUrlsTests(TestCase):
             self.assertNotIn('\x1b', self.output)
 
 
+class ListModelInfoTests(TestCase):
+    """
+    Tests for the `list_model_info` management command.
+    """
+    def test_plain(self):
+        out = StringIO()
+        call_command('list_model_info', '--model', 'django_extensions.MultipleFieldsAndMethods', stdout=out)
+        self.output = out.getvalue()
+        self.assertIn('id', self.output)
+        self.assertIn('char_field', self.output)
+        self.assertIn('integer_field', self.output)
+        self.assertIn('foreign_key_field', self.output)
+        self.assertIn('has_self_only()', self.output)
+        self.assertIn('has_one_extra_argument()', self.output)
+        self.assertIn('has_two_extra_arguments()', self.output)
+        self.assertIn('has_args_kwargs()', self.output)
+        self.assertIn('has_defaults()', self.output)
+        self.assertNotIn('__class__()', self.output)
+        self.assertNotIn('validate_unique()', self.output)
+
+    def test_all(self):
+        out = StringIO()
+        call_command('list_model_info', '--model', 'django_extensions.MultipleFieldsAndMethods', '--all', stdout=out)
+        self.output = out.getvalue()
+        self.assertIn('id', self.output)
+        self.assertIn('__class__()', self.output)
+        self.assertIn('validate_unique()', self.output)
+
+    @pytest.mark.skipif(sys.version_info < (3, 3), reason="inspect.Signature requires python3.3 or higher")
+    def test_signature(self):
+        out = StringIO()
+        call_command('list_model_info', '--model', 'django_extensions.MultipleFieldsAndMethods', '--signature', stdout=out)
+        self.output = out.getvalue()
+        self.assertIn('has_self_only(self)', self.output)
+        self.assertIn('has_one_extra_argument(self, arg_one)', self.output)
+        self.assertIn('has_two_extra_arguments(self, arg_one, arg_two)', self.output)
+        self.assertIn('has_args_kwargs(self, *args, **kwargs)', self.output)
+        self.assertIn("has_defaults(self, one=1, two='Two', true=True, false=False, none=None)", self.output)
+
+    def test_db_type(self):
+        if settings.DATABASES['default']['ENGINE'] == 'django.db.backends.postgresql':
+            id_type = 'serial'
+        else:
+            id_type = 'integer'
+
+        out = StringIO()
+        call_command('list_model_info', '--model', 'django_extensions.MultipleFieldsAndMethods', '--db-type', stdout=out)
+        self.output = out.getvalue()
+        self.assertIn('id - %s' % id_type, self.output)
+        self.assertIn('char_field - varchar(10)', self.output)
+        self.assertIn('integer_field - integer', self.output)
+        self.assertIn('foreign_key_field - integer', self.output)
+
+    def test_field_class(self):
+        out = StringIO()
+        call_command('list_model_info', '--model', 'django_extensions.MultipleFieldsAndMethods', '--field-class', stdout=out)
+        self.output = out.getvalue()
+        self.assertIn('id - AutoField', self.output)
+        self.assertIn('char_field - CharField', self.output)
+        self.assertIn('integer_field - IntegerField', self.output)
+        self.assertIn('foreign_key_field - ForeignKey', self.output)
+
+
 class MergeModelInstancesTests(TestCase):
     """
     Tests for the `merge_model_instances` management command.
@@ -256,7 +336,7 @@ class MergeModelInstancesTests(TestCase):
     @mock.patch('django_extensions.management.commands.merge_model_instances.apps.get_models')
     @mock.patch('django_extensions.management.commands.merge_model_instances.input')
     def test_get_model_to_merge(self, test_input, get_models):
-        class Model(object):
+        class Model:
             __name__ = ""
 
         return_value = []
@@ -272,13 +352,13 @@ class MergeModelInstancesTests(TestCase):
     @mock.patch('django_extensions.management.commands.merge_model_instances.input')
     def test_get_field_names(self, test_input):
 
-        class Field(object):
+        class Field:
             name = ""
 
             def __init__(self, name):
                 self.name = name
 
-        class Model(object):
+        class Model:
             __name__ = ""
             one = Field(name="one")
             two = Field(name="two")
