@@ -1,8 +1,11 @@
 # -*- coding: utf-8 -*-
-import mock
-
 from django_extensions.management.jobs import DailyJob
 
+try:
+    from unittest import mock
+except ImportError:
+    import mock
+
 
 DAILY_JOB_MOCK = mock.MagicMock()
 
