@@ -1,5 +1,9 @@
 # -*- coding: utf-8 -*-
+import django
+
 from django.db import models
+from django.contrib.auth import get_user_model
+from django.db.models.signals import pre_save
 
 from django_extensions.db.fields import AutoSlugField, ModificationDateTimeField, RandomCharField, ShortUUIDField
 from django_extensions.db.fields.json import JSONField
@@ -7,6 +11,9 @@ from django_extensions.db.models import ActivatorModel, TimeStampedModel
 
 from .fields import UniqField
 
+if django.VERSION >= (2, 2):
+    from django.db.models import UniqueConstraint
+
 
 class Secret(models.Model):
     name = models.CharField(blank=True, max_length=255, null=True)
@@ -48,7 +55,8 @@ class Person(models.Model):
     personality = models.OneToOneField(
         Personality,
         null=True,
-        on_delete=models.CASCADE)
+        on_delete=models.CASCADE,
+    )
     clubs = models.ManyToManyField(Club, through='testapp.Membership')
 
     class Meta:
@@ -93,7 +101,6 @@ class ThirdDummyRelationModel(models.Model):
 
 
 class PostWithUniqField(models.Model):
-
     uniq_field = UniqField(
         max_length=255,
         boolean_attr=True,
@@ -111,8 +118,7 @@ class PostWithUniqField(models.Model):
 
 
 class ReverseModel(models.Model):
-    post_field = models.ForeignKey(
-        PostWithUniqField, related_name='reverse_models', on_delete=models.CASCADE)
+    post_field = models.ForeignKey(PostWithUniqField, related_name='reverse_models', on_delete=models.CASCADE)
 
     class Meta:
         app_label = 'django_extensions'
@@ -146,6 +152,43 @@ class SluggedTestModel(models.Model):
         app_label = 'django_extensions'
 
 
+class SluggedWithConstraintsTestModel(models.Model):
+    title = models.CharField(max_length=42)
+    slug = AutoSlugField(populate_from='title')
+    category = models.CharField(max_length=20, null=True)
+
+    class Meta:
+        app_label = 'django_extensions'
+        if django.VERSION >= (2, 2):
+            constraints = [
+                UniqueConstraint(
+                    fields=['slug', 'category'],
+                    name="unique_slug_and_category",
+                ),
+            ]
+
+
+class SluggedWithUniqueTogetherTestModel(models.Model):
+    title = models.CharField(max_length=42)
+    slug = AutoSlugField(populate_from='title')
+    category = models.CharField(max_length=20, null=True)
+
+    class Meta:
+        app_label = 'django_extensions'
+        unique_together = ['slug', 'category']
+
+
+class OverridedFindUniqueAutoSlugField(AutoSlugField):
+    def find_unique(self, model_instance, field, iterator, *args):
+        self.overrided = True
+        return super().find_unique(model_instance, field, iterator, *args)
+
+
+class OverridedFindUniqueModel(models.Model):
+    title = models.CharField(max_length=42)
+    slug = OverridedFindUniqueAutoSlugField(populate_from='title')
+
+
 class CustomFuncSluggedTestModel(models.Model):
 
     def slugify_function(self, content):
@@ -340,6 +383,15 @@ class RandomCharTestModelPunctuation(models.Model):
         app_label = 'django_extensions'
 
 
+class RandomCharTestModelUniqueTogether(models.Model):
+    random_char_field = RandomCharField(length=8)
+    common_field = models.CharField(max_length=10)
+
+    class Meta:
+        app_label = 'django_extensions'
+        unique_together = ('random_char_field', 'common_field')
+
+
 class TimestampedTestModel(TimeStampedModel):
     class Meta:
         app_label = 'django_extensions'
@@ -347,9 +399,12 @@ class TimestampedTestModel(TimeStampedModel):
 
 class UnicodeVerboseNameModel(models.Model):
     cafe = models.IntegerField(verbose_name=u'café')
-    parent_cafe = models.ForeignKey('self', related_name='children',
-                                    on_delete=models.CASCADE,
-                                    verbose_name='café latte')
+    parent_cafe = models.ForeignKey(
+        'self',
+        related_name='children',
+        on_delete=models.CASCADE,
+        verbose_name='café latte',
+    )
 
     class Meta:
         app_label = 'django_extensions'
@@ -406,3 +461,42 @@ class DisabledUpdateModelModificationDateTimeField(models.Model):
 
     class Meta:
         app_label = 'django_extensions'
+
+
+class HasOwnerModel(models.Model):
+    content = models.TextField(default="")
+    owner = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)
+
+    class Meta:
+        app_label = 'django_extensions'
+
+
+class MultipleFieldsAndMethods(models.Model):
+    char_field = models.CharField(max_length=10)
+    integer_field = models.IntegerField()
+    foreign_key_field = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)
+
+    def has_self_only(self):
+        pass
+
+    def has_one_extra_argument(self, arg_one):
+        pass
+
+    def has_two_extra_arguments(self, arg_one, arg_two):
+        pass
+
+    def has_args_kwargs(self, *args, **kwargs):
+        pass
+
+    def has_defaults(self, one=1, two='Two', true=True, false=False, none=None):
+        pass
+
+    class Meta:
+        app_label = 'django_extensions'
+
+
+def dummy_handler(sender, instance, **kwargs):
+    pass
+
+
+pre_save.connect(dummy_handler, sender=HasOwnerModel)
