@@ -68,10 +68,8 @@ BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__fil
 STATIC_URL = "/static/"
 
 SHELL_PLUS_SUBCLASSES_IMPORT_MODULES_BLACKLIST = [
-    'django_extensions.db.fields.encrypted',
     'django_extensions.mongodb.fields',
     'django_extensions.mongodb.models',
-    'tests.test_encrypted_fields',
     'tests.testapp.scripts.invalid_import_script',
     'setup',
 ]
@@ -84,3 +82,16 @@ CACHES = {
         'BACKEND': 'tests.management.commands.test_clear_cache.OtherCacheMock',
     },
 }
+
+SHELL_PLUS_PRE_IMPORTS = [
+    'import sys, os',
+]
+SHELL_PLUS_IMPORTS = [
+    'from django_extensions import settings as django_extensions_settings',
+]
+SHELL_PLUS_POST_IMPORTS = [
+    'import traceback',
+    'import pprint',
+    'import os as test_os',
+    'from django_extensions.utils import *',
+]
